﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraController : MonoBehaviour 
{
	private bool isDragging = false; //Check if we are draging
	private Transform playerTransform; //Translation game object
	public GameObject targetObject; //Rotation game object
	public Vector2 dragSpeed; //drag speed spilt up for seperate control of x and y speeds (since camera is tilted x can see more than it would without the tilt)
	public float maxCameraDistance = 100f; 
	public float minCameraDistance = -100f;

	private bool isRotating = false; //Prevent the movement from happening while rotating
	private int currentRotation = 0; //Current rotation of camera
	private Quaternion[] goals = new Quaternion[] //Create an array of "Rotation points" that we will use for where to rotate to
	{
		Quaternion.Euler (new Vector3 (0, 0, 0)),
		Quaternion.Euler (new Vector3 (0, 90, 0)),
		Quaternion.Euler (new Vector3 (0, 180, 0)),
		Quaternion.Euler (new Vector3 (0, 270, 0))
	};

    void Start(){
		playerTransform = GetComponent<Transform>(); 
    }

    void Update() {

		if (Input.GetMouseButtonDown (0) && !isRotating) //Are we dragging and are we not rotating?
		{
			isDragging = true;
			StartCoroutine(dragRoutine());
		} 
		else if (Input.GetMouseButtonUp(0)) //When we aren't dragging and we are not rotating, reset for the next time
		{
			isDragging = false;
		}
    }

	private IEnumerator dragRoutine()
	{
		Vector3 touchStartPos = Camera.main.ScreenToWorldPoint(new Vector3 (Input.mousePosition.x * dragSpeed.x,Input.mousePosition.y * dragSpeed.y,1000f));

		Vector3 TouchPos;
		Vector3 CameraStart = targetObject.transform.position;

		while (isDragging) 
		{
			TouchPos = Camera.main.ScreenToWorldPoint (new Vector3 (Input.mousePosition.x * dragSpeed.x,Input.mousePosition.y * dragSpeed.y,1000f)) - (targetObject.transform.position-CameraStart);

			targetObject.transform.position = 
				new Vector3 ((Mathf.Clamp((CameraStart.x + touchStartPos.x - TouchPos.x),minCameraDistance,maxCameraDistance)),
				CameraStart.y,
				Mathf.Clamp((CameraStart.z + touchStartPos.z - TouchPos.z),minCameraDistance,maxCameraDistance)); // Moves the camera from it's starting position to where we started touching, minus where we are currently touching. Movement clamped in both the x and z axis' to create a perfect square/rectangle for camera view field.


			yield return null;
		}
	}
      
	[ContextMenu("Left")] //Show within the context menu within the editor for testing purposes
	public void LeftBtn(){
		StartCoroutine (RotateRoutine (false, 1));
	}

	[ContextMenu("Right")] //Show within the context menu within the editor for testing purposes
	public void RightBtn(){
		StartCoroutine (RotateRoutine (true, 1));
	}

	private int newPosition (int _start, bool _right, int _steps = 1) 
	{
		int newRotation = _start;
		for (int i = 0; i < _steps; i++) {
			if (!_right) {
				if (newRotation < goals.Length - 1) {
					newRotation++;
				} else {
					newRotation = 0;
				}
			} else {
				if (newRotation > 0) {
					newRotation--;
				} else {
					newRotation = goals.Length - 1;
				}
			}
		}
		return newRotation;
	}
		
	private IEnumerator RotateRoutine(bool _right, float _duration)
	{
		isRotating = true;
		//goals
		Quaternion start = goals [currentRotation];
		currentRotation = newPosition (currentRotation,_right);
		Quaternion end = goals [currentRotation];
		//move
		float count = _duration;
		float blend;
		while (count > 0) {
			count -= Time.deltaTime;
			 
			blend = (Mathf.Cos(((1f / _duration) * count) * Mathf.PI) + 1f)/2f;
			targetObject.transform.rotation = Quaternion.Lerp (start, end, blend);

			yield return null;
		}
		targetObject.transform.rotation = end;
		isRotating = false;
	}
}