﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CharacterCreationManager : MonoBehaviour
{
	[SerializeField] MasterCharacterManager _characterPrefab;
	Stack<MasterCharacterManager> _disabledCharacters = new Stack<MasterCharacterManager> ();
	int _disabledCharacterCount = 0;

	// infected count management
	[SerializeField]
	private int maxInfectedCount = 20;
	private int infectedCharactersCount;

	// references
	[HideInInspector]
	public List<Transform> spawnPoints = new List<Transform> ();
	public List<MasterCharacterManager> activeCharacters = new List<MasterCharacterManager>();
	public List<MasterCharacterManager> inactiveCharacters = new List<MasterCharacterManager>();

	#region Infected Count Functions

	public int GetInfectedCount()
	{
		return infectedCharactersCount;
	}

	public void ResetInfectedCount()
	{
		infectedCharactersCount = 0;
	}

	public void IncreaseInfectedCount()
	{
		infectedCharactersCount++;
		if (infectedCharactersCount >= maxInfectedCount)
		{
			MasterGameController.Instance.BeginDefeatPhase();
		}
	}

	#endregion

	#region Spawn Point Functions

	public void SetSpawnPoints(GameTile[] cityTiles)
	{
		// clear the spawn points
		spawnPoints.Clear();


		// add all of the spawn points
		for (int i = 0; i < cityTiles.Length; i++)
		{
			for (int j = 0; j < cityTiles[i].SpawnPoints.Length; j++)
			{
				spawnPoints.Add(cityTiles[i].SpawnPoints[j]);
			}
		}
	}

	#endregion

	#region Character List Functions

	public void SetCharacters(GameTile[] cityTiles)
	{
		// clear the lists
		inactiveCharacters.Clear();
		activeCharacters.Clear();

		// add all of the characters
		for (int i = 0; i < cityTiles.Length; i++)
		{
			for (int j = 0; j < cityTiles[i].characters.Length; j++)
			{
//				cityTiles[i].characters[j].gameObject.SetActive(false);
				inactiveCharacters.Add(cityTiles[i].characters[j]);
			}
		}
	}

	#endregion

	#region Character Management Functions

	public MasterCharacterManager CreateInfected()
	{
		int spawnIndex = Random.Range(0, spawnPoints.Count);
		int destIndex = 0;
		do
		{
			destIndex = Random.Range(0, spawnPoints.Count);
		} while (destIndex == spawnIndex);

		MasterCharacterManager character = CreateCharacter(spawnIndex, destIndex);
		if (character != null)
		{
			character.SetInfected();
			character.InfectedSearching();
		}
		return character;
	}

	public MasterCharacterManager CreateCharacter()
	{ 
		int spawnIndex = Random.Range(0, spawnPoints.Count);
		int destIndex = 0;
		do
		{
			destIndex = Random.Range(0, spawnPoints.Count);
		} while (destIndex == spawnIndex);

		return CreateCharacter(spawnIndex, destIndex);
	}

	public MasterCharacterManager CreateCharacter(int spawnIndex, int destIndex)
	{ 
		MasterCharacterManager character = null;
		if (inactiveCharacters.Count > 0)
		{
			// Set the position of this person. Could also set rotation
			character = inactiveCharacters[0];
			character.gameObject.SetActive(true);
			character.SetNormie(spawnPoints[destIndex].position);
			character.gameObject.transform.SetPositionAndRotation(spawnPoints[spawnIndex].position, spawnPoints[spawnIndex].rotation);

			// switch which list the character is on
			character.activeCharacterIndex = activeCharacters.Count;
			activeCharacters.Add(inactiveCharacters[0]);
			inactiveCharacters.RemoveAt(0);

			Debug.Log("Spawned Character", character);
		}

		// Return the character
		return character;
	}

	public void DisableCharacter(MasterCharacterManager character)
	{
		character.gameObject.SetActive(false);
		inactiveCharacters.Add(character);
		activeCharacters.RemoveAt(character.activeCharacterIndex);
		character.activeCharacterIndex = 0;
	}

	#endregion

}
