﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class NormieCharacterManager : CharacterDecisionManager
{
	// Normies will be scared for this long before becoming bored again
	const float HEALTHY_FEAR_AMOUNT = 4.0f;
	float _fearStrength;
	float _desiredLocationScaleFactor = 200.0f;
	Vector2 _destination;

	public enum NormieLogic
	{
		idle,
		bored,
		scared
	}

	#region Initialization Functions

	GameObject _destinationBuilding;
	NormieLogic _currNormieMentality;

	public override void Initialize()
	{
		base.Initialize();

		_fearStrength = HEALTHY_FEAR_AMOUNT;

		// set states
		showVisionCone = false;
		_currNormieMentality = NormieLogic.idle;

//		StartCoroutine(MoveToDest());
	}

	#endregion

//	IEnumerator MoveToDest()
//	{
//		yield return null;
//
//		Debug.LogFormat("Heading over to {0}", _destination.ToString());	
//		characterManager.GoToPosition(_destination);
//	}

	public override void RunUpdate()
	{
		base.RunUpdate();

		if (_currNormieMentality == NormieLogic.bored)
		{
			// Are we near infected we haven't seen before? if so, set our destination to away from them
			// Otherwise, 
			// Do we have a destination? if not, request a destination
			// Are we near our destination? if so, maybe like, enter it or something

			// determine if the normie can see an infected attacking a creature
			MasterCharacterManager infectedSeen = characterManager.visionManager.GetClosestVisibleCharacter(characterVisionMask);
			bool canSeeInfected = false;
			if (infectedSeen != null)
			{
				if (infectedSeen.GetComponent<InfectedCharacterManager>().currentInfectedMentality == InfectedCharacterManager.InfectedLogic.infecting)
				{
					canSeeInfected = true;
				}
			}

			// 
			if (canSeeInfected)
			{
				// Run away for a while 
				StartCoroutine(runAwayFromInfected(
					infectedSeen, 
					_fearStrength));
				
			}
			else if (reachedDestination())
			{
				MasterGameController.Instance.characterCreationManager.DisableCharacter(characterManager);
			}
			else
			{
				characterManager.GoToPosition(_destination);
			}
		}

	}

	// TODO - maybe get a location in the opposite direction from the infected, so the fleeing logic makes sense
	private IEnumerator runAwayFromInfected(MasterCharacterManager infectedCharacter, float runningAwayTime)
	{
		_currNormieMentality = NormieLogic.scared;

		var fearDest = getFearDestination(infectedCharacter);
		characterManager.GoToPosition(fearDest);

		yield return new WaitForSeconds (runningAwayTime);

		// ancient normie-logic dictates the danger has now passed
		_currNormieMentality = NormieLogic.bored;
		characterManager.GoToPosition(_destination);
	}

	private Vector3 getFearDestination(MasterCharacterManager character)
	{
		var seenPersonPosition = character.transform.position;
		var myPosition = transform.position;
		
		// Run away in the opposite direction and go really far
		var myDesiredPosition = (myPosition - seenPersonPosition).normalized * (_desiredLocationScaleFactor)
		                        + myPosition;

		return myDesiredPosition;
	}

	public void SetDestination(Vector3 dest)
	{
		_destination = new Vector2 (dest.x, dest.z);	
		_currNormieMentality = NormieLogic.bored;
	}

	private bool reachedDestination()
	{
		// TODO - check if arrived at destination
		var pos = transform.position;
		var flatPos = new Vector2 (pos.x, pos.z);

		// 
		return Vector2.Distance(flatPos, _destination) < CLOSE_ENOUGH_TO_DEST;
	}
}
