﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class MasterGameController : MonoBehaviour
{
	[Header("Controller References")]
	public TimeController timeController;
	public StreetLightContoller streetLightController;
	public CharacterCreationManager characterCreationManager;
	public BoardGeneration boardGeneration;

	// Boundary Locations
	[Header("Boundaries")]
	public List<DetectableObject> boundaryLocations;

	// Game State
	public enum GamePhase
	{
		idle,
		active,
		victory,
		defeat
	}
	[HideInInspector]
	public GamePhase currentGamePhase;

	public int normiesToSpawn = 300;

	#region Instance Functions

	// Instance
	private static MasterGameController instance;

	public static MasterGameController Instance {
		get {
			return instance;
		}
	}

	private void SetInstance()
	{
		// set the instance
		if (instance != null)
		{
			Destroy(instance.gameObject);
		}
		instance = this;
	}

	#endregion

	#region Mono Functions

	private void Awake()
	{
		SetInstance();
		BeginInitializePhase();
	}

	#endregion

	#region Game Phase Functions

	public void BeginInitializePhase()
	{
		currentGamePhase = GamePhase.idle;
	}

	[ContextMenu("Begin Game")]
	public void BeginActivePhase()
	{
		currentGamePhase = GamePhase.active;

		// set the timer
//		CreateTimeMilestones();
		timeController.GameDuration = 60;
		timeController.ResetGameTimer();
		timeController.StartGameTimer();
		ResumeGame();
	}

	public void BeginDefeatPhase()
	{
		currentGamePhase = GamePhase.defeat;

		// pause the game
		PauseGame();
	}

	public void BeginVictoryPhase(int val)
	{
		currentGamePhase = GamePhase.victory;

		// pause the game
		PauseGame();
	}

	public void PauseGame()
	{
		Time.timeScale = 0;
	}

	public void ResumeGame()
	{
		Time.timeScale = 1;
	}

	#endregion

	#region Timer Functions

	private void CreateTimeMilestones()
	{
		// create infected spawning times
		timeController.AddTimeMilestone(1, 1, SpawnInfected);
		timeController.AddTimeMilestone(30, 2, SpawnInfected);
		timeController.AddTimeMilestone(90, 3, SpawnInfected);
		timeController.AddTimeMilestone(180, 5, SpawnInfected);
		timeController.AddTimeMilestone(280, 8, SpawnInfected);

		// add normies
		int remainingNormies = normiesToSpawn;
		int normiesSpawned = 0;
		for (int i = 0; i < 30; i++)
		{
			if (remainingNormies <= 0)
			{
				break;
			}
			timeController.AddTimeMilestone((10 * i) + 1, Random.Range(1, Mathf.Min(10, remainingNormies)), SpawnNormie);
		}
		if (remainingNormies < 0)
		{
			timeController.AddTimeMilestone(310, remainingNormies, SpawnNormie);
		}

		// set up the end game
		timeController.AddTimeMilestone((int)timeController.GameDuration, 0, BeginVictoryPhase);
	}

	#endregion

	#region Character Spawner Functions

	public void SpawnNormie(int val)
	{
		for (int i = 0; i < val; i++)
		{
			characterCreationManager.CreateCharacter();
		}
	}

	public void SpawnInfected(int val)
	{
		if (val > characterCreationManager.GetInfectedCount())
		{
			for (int i = characterCreationManager.GetInfectedCount(); i < val; i++)
			{
				characterCreationManager.CreateInfected();
			}
		}
	}

	#endregion

	#region Boundary Functions

	public DetectableObject SelectRandomBoundaryLocation()
	{
		return boundaryLocations[Random.Range(0, boundaryLocations.Count)];
	}

	#endregion
}
