﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class TimeController : MonoBehaviour
{
	public class TimeMilestone
	{
		public int time;
		public int val;
		public Action<int> triggerEvent;
	}

	// Time keeping variables
	private List<TimeMilestone> timeMilestones = new List<TimeMilestone>();
	private float currentTime;
	private float gameDuration;

	// Trigger Variables
	private bool isActive;

	#region Time Keeping Functions

	public float GameDuration {
		get {
			return gameDuration;
		}
		set {
			gameDuration = value;
		}
	}

	public void StartGameTimer()
	{
		isActive = true;
	}

	public void PauseGameTimer()
	{
		isActive = false;
	}

	public void ResetGameTimer()
	{
		currentTime = 0;
	}

	public void StopGameTimer()
	{
		PauseGameTimer();
		ResetGameTimer();
	}

	public float GetCurrentTime()
	{
		return currentTime;
	}

	#endregion

	#region Time Milestone Functions

	public void AddTimeMilestone(int time, int val, Action<int> trigger)
	{
		// create the time milestone
		TimeMilestone tm = new TimeMilestone ();
		tm.time = time;
		tm.val = val;
		tm.triggerEvent = trigger;
		timeMilestones.Add(tm);

		// sort the events
		timeMilestones.Sort(delegate(TimeMilestone a, TimeMilestone b) {
			return a.time.CompareTo(b.time);
		});
	}

	private void TriggerTimeMilestone()
	{
		while (timeMilestones.Count > 0)
		{
			// check if we passed the milestone
			if (currentTime > timeMilestones[0].time)
			{
				timeMilestones[0].triggerEvent(timeMilestones[0].val);
				timeMilestones.RemoveAt(0);
			}
			else
			{
				break;
			}
		}
	}

	#endregion

	#region Mono Functions

	private void Update()
	{
		if (isActive)
		{
			currentTime += Time.deltaTime;
			TriggerTimeMilestone();
		}
	}

	#endregion
}
