

Shader "Custom/Pedestrian" {
    Properties {
        _Shadow ("Shadow Color", Color) = (0,0,0,0) 
        _Light("Light Color", Color) = (1,1,1,1) 
       _Blend ("blend", Range (0, 1)) = 0
        [NoScaleOffset] _MainTex ("Normal Diffuse", 2D) = "white" {}
        [NoScaleOffset] _InfectedTex ("EInfected Diffuse", 2D) = "white" {}
    }
    SubShader {
        Tags { "RenderType"="Opaque" "Queue"="Geometry" }
		LOD 100

        //Base Pass (We Build on this its just black)
        Pass
		{
			CGPROGRAM
			#pragma vertex vert
			#pragma fragment frag

			struct appdata{
				float4 vertex : POSITION;
			};

			struct v2f{
				float4 vertex : SV_POSITION;
			};

			v2f vert (appdata v){
				v2f o;
				o.vertex = UnityObjectToClipPos(v.vertex);
				return o;
			}
			
			fixed4 frag (v2f i) : SV_Target{
				return half4(0,0,0,1);
			}

			ENDCG
		}
        //Directional Light 
        Pass
        {
            Tags {"LightMode"="ForwardBase"}
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag

            #pragma multi_compile_fwdbase nolightmap nodirlightmap nodynlightmap novertexlight
            #include "UnityCG.cginc"
            #include "Lighting.cginc"
            #include "AutoLight.cginc"
            #pragma multi_compile_fog

            struct v2f {
                float2 uv : TEXCOORD0;
                SHADOW_COORDS(1)
                UNITY_FOG_COORDS(2)
                fixed3 diff : COLOR0;
                fixed3 ambient : COLOR1;
                float4 pos : SV_POSITION;
            };

            v2f vert (appdata_base v)
            {
                v2f o;
                o.pos = UnityObjectToClipPos(v.vertex);
                o.uv = v.texcoord;
                half3 worldNormal = UnityObjectToWorldNormal(v.normal);
                half nl = max(0, dot(worldNormal, _WorldSpaceLightPos0.xyz));
                o.diff = nl * _LightColor0.rgb;
                o.ambient = ShadeSH9(half4(worldNormal,1));
                UNITY_TRANSFER_FOG(o,o.pos);
                TRANSFER_SHADOW(o)
                return o;
            }

            sampler2D _MainTex;
            sampler2D _InfectedTex;
            half4 _Shadow;
            half4 _Light;
            half _Blend;

            fixed4 frag (v2f i) : SV_Target
            {

                fixed4 mainTex = tex2D(_MainTex, i.uv) * (1 - _Blend) + tex2D(_InfectedTex, i.uv) * _Blend;
                fixed atten = SHADOW_ATTENUATION(i) ;
                fixed3 lighting = i.diff * atten;
                
                //Light
                half4 color = _Shadow * (1- atten) + mainTex * atten * half4(lighting,1) * i.ambient.g;
                UNITY_APPLY_FOG(i.fogCoord, color);
                return color;
            }
            ENDCG
        }


		//Point Lights (Fuck them Monsters)
        Pass {
            Tags { "LightMode" = "ForwardAdd" }
            Blend One One
            Fog { Color(0,0,0,0) }
 
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
            #pragma multi_compile_fwdadd_fullshadows
            #include "UnityCG.cginc"
            #include "AutoLight.cginc"

			struct appdata{
				float4 vertex : POSITION;
				float2 uv : TEXCOORD0;
			};

            struct v2f {
                float4 pos : SV_POSITION;
				float2 uv : TEXCOORD0;
                LIGHTING_COORDS(1,2)
            };

 			sampler2D _MainTex;
            sampler2D _InfectedTex;
            half4 _Shadow;
            half _Blend;
			
            v2f vert (appdata v) {
                v2f o;
                o.pos = UnityObjectToClipPos(v.vertex);
				o.uv = v.uv;
                TRANSFER_VERTEX_TO_FRAGMENT(o);
                return o;
            }
 
            float4 frag (v2f i) : COLOR {
                fixed4 mainTex = tex2D(_MainTex, i.uv) * (1 - _Blend) + tex2D(_InfectedTex, i.uv) * _Blend;

                //Light
                half4 atten = LIGHT_ATTENUATION(i);
                half lightMask = clamp(0, 1 , (atten.r + atten.g + atten.b) / 3) *2 ;

                half4 color = mainTex * lightMask;
		
                return color;
            }
            ENDCG
        } //Pass
    } //SubShader
    FallBack "Diffuse" //note: for passes: ForwardBase, ShadowCaster, ShadowCollector
}